/*******************************************************************************
 * Copyright (c) 2003, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     Matt Chapman - initial version
 *******************************************************************************/
package org.eclipse.contribution.visualiser.cvsprovider;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.contribution.visualiser.interfaces.IContentProvider;
import org.eclipse.contribution.visualiser.interfaces.IGroup;
import org.eclipse.contribution.visualiser.interfaces.IMember;
import org.eclipse.jface.resource.ImageDescriptor;

/**
 * Simple implementation of a content provider
 */
public class CVSContentProvider implements IContentProvider {

	private static List groups = null;
	

	/**
	 * Returns all registered groups
	 * @see org.eclipse.contribution.visualiser.IVisContentProvider#getAllGroups()
	 * @return List of IGroups
	 */
	public List getAllGroups() {
		return groups;
	}


	/**
	 * Returns all IMembers contained in the given IGroup 
	 * @see org.eclipse.contribution.visualiser.IVisContentProvider#getAllMembers(org.eclipse.contribution.visualiser.IGroup)
	 */
	public List getAllMembers(IGroup group) {
		return group.getMembers();
	}
	
	
	/**
	 * Returns the List of all IMembers in all registered groups
	 */
	public List getAllMembers() {
		List grps = getAllGroups();
		List members = new ArrayList();
		if (grps == null) return members;
		Iterator iter = grps.iterator();
		while (iter.hasNext()) {
			IGroup grp = (IGroup)iter.next();
			List membersInGroup = getAllMembers(grp);
			Iterator iter2 = membersInGroup.iterator();
			while (iter2.hasNext()) {
				IMember im = (IMember)iter2.next();
				members.add(im);
			}
		}
		return members;
	}


	/**
	 * Initialise the content provider.  This is a default imploementation and
	 * so does nothing.
	 * @see org.eclipse.contribution.visualiser.interfaces.IContentProvider#initialise()
	 */
	public void initialise() {
	}

	public static void clearGroups() {
		groups = null;	
	}
	
	/**
	 * Register a group
	 * @param grp
	 */
	public static void addGroup(IGroup grp) {
	  if (groups == null) groups = new ArrayList();
	  groups.add(grp);
	}
	
	
	/**
	 * Get the total number of groups registered
	 * @return
	 */
	public int numberOfGroupsDefined() {
		return (groups==null?0:groups.size());
	}


	/**
	 * Process a mouse click on a member belonging to this provider.  This is a default
	 * implementation and so does nothing and returns true to allow the visualiser
	 * to perform it's default mouse click operation.
	 * @see org.eclipse.contribution.visualiser.interfaces.IContentProvider#processMouseclick(java.lang.String, boolean, int)
	 */
	public boolean processMouseclick(IMember member, boolean markupWasClicked,int buttonClicked) {
		return true;
	}
	
	public String getEmptyMessage() {
		return "";
	}


	/* (non-Javadoc)
	 * @see org.eclipse.contribution.visualiser.interfaces.IContentProvider#getMemberViewIcon()
	 */
	public ImageDescriptor getMemberViewIcon() {
		// TODO Auto-generated method stub
		return null;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.contribution.visualiser.interfaces.IContentProvider#getGroupViewIcon()
	 */
	public ImageDescriptor getGroupViewIcon() {
		// TODO Auto-generated method stub
		return null;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.contribution.visualiser.interfaces.IContentProvider#activate()
	 */
	public void activate() {
		// TODO Auto-generated method stub
		
	}


	/* (non-Javadoc)
	 * @see org.eclipse.contribution.visualiser.interfaces.IContentProvider#deactivate()
	 */
	public void deactivate() {
		// TODO Auto-generated method stub
		
	}
}
