/*******************************************************************************
 * Copyright (c) 2003, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     Matt Chapman - initial version
 *******************************************************************************/
package org.eclipse.contribution.visualiser.cvsprovider;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.eclipse.contribution.visualiser.VisualiserPlugin;
import org.eclipse.contribution.visualiser.core.ProviderManager;
import org.eclipse.contribution.visualiser.core.Stripe;
import org.eclipse.contribution.visualiser.interfaces.IGroup;
import org.eclipse.contribution.visualiser.interfaces.IMember;
import org.eclipse.contribution.visualiser.simpleImpl.SimpleGroup;
import org.eclipse.contribution.visualiser.simpleImpl.SimpleMarkupKind;
import org.eclipse.contribution.visualiser.simpleImpl.SimpleMember;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.internal.ccvs.core.CVSException;
import org.eclipse.team.internal.ccvs.core.CVSProviderPlugin;
import org.eclipse.team.internal.ccvs.core.CVSTeamProvider;
import org.eclipse.team.internal.ccvs.core.ICVSRemoteFile;
import org.eclipse.team.internal.ccvs.core.ICVSResource;
import org.eclipse.team.internal.ccvs.core.ILogEntry;
import org.eclipse.team.internal.ccvs.core.resources.CVSWorkspaceRoot;
import org.eclipse.team.internal.ccvs.ui.actions.WorkspaceAction;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchWindow;

public class ShowInVisualiserAction
	extends WorkspaceAction
	implements IObjectActionDelegate {

	private ILogEntry currentEntry;
	private Shell shell = new Shell();

	private CVSMarkupProvider markupProvider;
	
	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void execute(IAction action)
		throws InterruptedException, InvocationTargetException {
	
		IRunnableWithProgress process = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException {
				IResource[] resources = getSelectedResources();
				if (resources.length != 1)
					return;
				showVersionTree(monitor,resources[0]);
				//VersionTreeView view = (VersionTreeView) showView(VersionTreeView.VIEW_ID);
				//if (view != null) {
				//	view.showVersionTree(resources[0]);
				//	}
			}
		};
		//, false /* cancelable */
		//, PROGRESS_BUSYCURSOR);

		Shell shell = null;
		IWorkbenchWindow activeWindow = CVSVisualiserPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow();	
		if(activeWindow != null){
			shell = activeWindow.getShell();
		}
		final ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(shell){
			protected void configureShell(Shell shell){
				super.configureShell(shell);
				shell.setText("Querying CVS");
			}
		};
		progressDialog.run(false, true, process);	
		
	}

	protected boolean isEnabled() throws TeamException {
		return true;
	}

	public void showVersionTree(IProgressMonitor monitor, IResource resource) {
//		System.out.println("show=" + resource);
		if (resource != null) {
			CVSContentProvider.clearGroups();
			markupProvider = (CVSMarkupProvider)ProviderManager.getMarkupProvider();
			markupProvider.initialise();
			VisualiserPlugin.menu.reset();

			if (resource instanceof IFile) {
				IFile file = (IFile) resource;
				handleFile(monitor, file);
			} else if (resource instanceof IFolder) {
				IFolder folder = (IFolder)resource;
				try {
					IResource[] children = folder.members();
					monitor.beginTask("Querying CVS...",children.length*10);
					for (int i=0; i<children.length; i++) {
						if (children[i] instanceof IFile) {
							SubProgressMonitor subMonitor= new SubProgressMonitor(monitor, 10);
							handleFile(subMonitor, (IFile)children[i]);
						}
					}
				} catch (CoreException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				monitor.done();
			}
			markupProvider.processMarkups();
			VisualiserPlugin.menu.ensureUptodate();
			VisualiserPlugin.refresh();
		}
	}

	/**
	 * Shows the version tree for the given IResource in the view.
	 * 
	 * Only files are supported for now.
	 */
	public void handleFile(IProgressMonitor monitor, IFile file) {
//		System.out.println("handle file=" + file);

		RepositoryProvider teamProvider =
			RepositoryProvider.getProvider(
				file.getProject(),
				CVSProviderPlugin.getTypeId());
		if (teamProvider != null) {
			CVSTeamProvider provider = (CVSTeamProvider) teamProvider;
			try {
				// for a file this will return the base
				ICVSRemoteFile remoteFile =
					(ICVSRemoteFile) CVSWorkspaceRoot.getRemoteResourceFor(
						file);
				final ILogEntry[] logs = getLogEntries(monitor, remoteFile);

				String parent = file.getParent().getName();
				IGroup group = new SimpleGroup(parent);
				CVSContentProvider.addGroup(group);

				Date first = logs[logs.length-1].getDate();
				long now = System.currentTimeMillis();
				long msecs = now - first.getTime();
				int days = (int)(msecs / (1000*60*60*24)); // no. ms in a day 
//				System.out.println("age = "+days+" days");

				IMember mem = new SimpleMember(file.getName());
				mem.setSize(days);
				group.add(mem);

				for (int i=0; i<logs.length; i++) {
					msecs = now - logs[i].getDate().getTime();
					days = (int)(msecs / (1000*60*60*24)); // no. ms in a day 
					//Stripe s = new Stripe(logs[i].getAuthor(),days,1);
					List l = new ArrayList();
					l.add(new SimpleMarkupKind(logs[i].getAuthor()));
					Stripe s = new Stripe(l,days,1);
					//s.setToolTip(logs[i].getRevision()+", "+logs[i].getAuthor()+", "
					//	+logs[i].getDate()+"\n"+logs[i].getComment());
					markupProvider.addMarkup(parent+"."+file.getName(),s);
				}

			} catch (TeamException e) {
				System.err.println(e);
				//CVSUIPlugin.openError(
				//	getViewSite().getShell(),
				//	null,
				//	null,
				//	e);
			}
		}
		return;

	}

	private ILogEntry[] getLogEntries(IProgressMonitor monitor, final ICVSRemoteFile remoteFile) {
		String currentRevision = ""; //$NON-NLS-1$
		currentEntry = null;
		try {
			currentRevision = remoteFile.getRevision();
		} catch (TeamException e1) {
		}
		final ILogEntry[][] result = new ILogEntry[1][];
		try {
				ILogEntry[] entries = remoteFile.getLogEntries(monitor);
				result[0] = entries;
		} catch (TeamException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		// set current revision
		for (int i = 0; i < result[0].length; i++) {
			ILogEntry entry = result[0][i];
//			System.out.println(
//				"entry["
//					+ i
//					+ "]="
//					+ entry.getRevision()
//					+ ","
//					+ entry.getAuthor()
//					+ ","
//					+ entry.getDate()
//					+ ","
//					+ entry.getComment());
			if (entry.getRevision().equals(currentRevision))
				currentEntry = entry;
		}
		return result[0];
	}

	/**
	 * @see org.eclipse.team.internal.ccvs.ui.actions.WorkspaceAction#isEnabledForMultipleResources()
	 */
	protected boolean isEnabledForMultipleResources() {
		return false;
	}

	/**
	 * @see org.eclipse.team.internal.ccvs.ui.actions.WorkspaceAction#isEnabledForAddedResources()
	 */
	protected boolean isEnabledForAddedResources() {
		return false;
	}

	/**
	 * @see org.eclipse.team.internal.ccvs.ui.actions.WorkspaceAction#isEnabledForCVSResource(org.eclipse.team.internal.ccvs.core.ICVSResource)
	 */
	protected boolean isEnabledForCVSResource(ICVSResource cvsResource)
		throws CVSException {
		return (
			!cvsResource.isFolder()
				&& super.isEnabledForCVSResource(cvsResource));
	}

}
